program TestGZip;

{$mode objfpc}{$H+}

uses
  {$IFDEF UNIX}{$IFDEF UseCThreads}
  cthreads,
  {$ENDIF}{$ENDIF}
  Classes, SysUtils, Interfaces,

  IdStack,
  IdCompressorZLib ,
  IdZLib,
  IdZLibHeaders, pl_indy

  { you can add units after this };


function FavLoadFromFile(InFileName: String; InRaiseErrorIfNotExists: Boolean = False): String;
var MyFile: Text;
    MyFileAssigned: Boolean;
    MyBuffer: String;
    MyString: String;
begin
  MyFileAssigned:=False;
  MyString := '';

  if InRaiseErrorIfNotExists then begin
    if FileExists(InFileName) = false then begin
      raise Exception.Create('File: "' + InFileName + '" does not exists!');
    end;
  end;
  try
    AssignFile(MyFile, InFileName);
    MyFileAssigned:=True;
    Reset(MyFile);
    while not Eof(MyFile) do begin
      ReadLn(MyFile, MyBuffer);
      if MyString <> '' then MyString := MyString + #13#10;
      MyString := MyString + MyBuffer;
    end;
    CloseFile(MyFile);
    MyFileAssigned:=False;
  except
    on E: Exception do begin
      if MyFileAssigned then begin
        try
          CloseFile(MyFile);
        finally
        end;
      end;
      raise;
    end;
  end;
  Result := MyString;
end;

procedure FavAppendToFile(InFileName: String; const InString: String);
var MyFile: Text;
    MyFileAssigned: Boolean;
begin
  MyFileAssigned:=False;
  try
    AssignFile(MyFile, InFileName);
    MyFileAssigned:=True;
    if FileExists(InFileName) then begin
      Append(MyFile);
    end
    else begin
      Rewrite(MyFile);
    end;
    Write(MyFile, InString);
    Flush(MyFile);
    CloseFile(MyFile);
    MyFileAssigned:=False;
  except
    on E: Exception do begin
      if MyFileAssigned then begin
        try
          CloseFile(MyFile);
        finally
        end;
      end;
      raise;
    end;
  end;
end;

procedure FavStringToStream(OutStream: TStream; const InString: String);
begin
  try
    OutStream.Position:=0;

    if OutStream is TMemoryStream then begin
      OutStream.WriteBuffer(Pointer(InString)^, Length(InString));
    end
    else if OutStream is TFileStream then begin
      OutStream.WriteBuffer(Pointer(InString)^, Length(InString));
    end
    else begin
      OutStream.WriteBuffer(Pointer(InString)^, Length(InString));
    end;

    OutStream.Position:=0;
  except
    on E: Exception do begin
      raise (E);
    end;
  end;
end;

procedure FavStringToGZIP(OutStream: TStream; const InString: String);
var MyMemoryStream: TMemoryStream;
    MyIdCompressorZLib: TIdCompressorZLib;
    MyError: String;
begin
  MyMemoryStream := nil;
  MyIdCompressorZLib := nil;
  try
    MyMemoryStream := TMemoryStream.Create();
    FavStringToStream(MyMemoryStream, InString);

    // This is solution with INDY library which use zlib1.dll
    MyIdCompressorZLib := TIdCompressorZLib.Create(nil);
    // This is gzip compression
    //
    MyIdCompressorZLib.CompressStream(MyMemoryStream, OutStream, 9, GZIP_WINBITS, 9, 0);
//    MyIdCompressorZLib.CompressStream(MyMemoryStream, OutStream, 5, GZIP_WINBITS, 8, 0);

    if MyMemoryStream <> nil then FreeAndNil(MyMemoryStream);
    if MyIdCompressorZLib <> nil then FreeAndNil(MyIdCompressorZLib);
  except
    on E: Exception do begin
      MyError := E.ClassName + '; '+ E.Message;
      if MyMemoryStream <> nil then FreeAndNil(MyMemoryStream);
      if MyIdCompressorZLib <> nil then FreeAndNil(MyIdCompressorZLib);
      raise;
    end;
  end;
end;


procedure FavTestGZip;
var MyString: String;
    MyLength: Longint;
    MyCompress: String;
    MyMemoryStream: TMemoryStream;
    MyLengthCompress: Longint;
    F: Integer;
    MyCount: INteger;
begin
  MyMemoryStream := TMemoryStream.Create;
  MyString := FavLoadFromFile('uncompressable.xml');
  MyLength := Length(MyString);

  F := 1;
  F := 674350;  // critical is 674401 for level 9 and 8
  F := 1186350; // for level 7
  F := 1695550; // for level 6 and 5
  F := 2182850; // for level 4
  F := 2335850; // for level 1

  F := 675350;  // critical is 674401 for level 9 and 9
  F := 673350;  // little bit less for OK test
  while F < MyLength do begin
    FavAppendToFile('Log.log', FormatDateTime('yyyy-mm-dd hh:nn:ss.zzzzzz', Now) + '; Compressing String of: ' + IntToStr(F));
    MyMemoryStream.Clear;
    FavStringToGZIP(MyMemoryStream, Copy(MyString, 1, F));
//    FavStringToGZIP2(MyMemoryStream, Copy(MyString, 1, F));
    MyLengthCompress := MyMemoryStream.Size;
    FavAppendToFile('Log.log', '; Compressing OK: ' + IntToStr(MyLengthCompress) + #13#10);
    F := F + 1000;
  end;

  FavAppendToFile('Log.log', 'All OK' + #13#10);
  FreeAndNil(MyMemoryStream);
end;

// *********************************************
(*
The purpose of test is to show that Indy
library can not GZIP compress some string
String is saved in file uncompresable.xml
String is loaded into memory and
I cut peace of files and compresse it
After some time I found on peace which
cannot be compressed and program returns
zlib error -5 which means that buffer is
not good. If you change MemLevel from 9 to
lower values or compression level from 9 to
no lower values you can compress this string
but eventually if you try some other strings
you get the same result (-5).
Indy use zlib1.dll (ver 1.2.8) for 32 bit windows
and zlibwapi.dll (ver 1.2.5) for 64 bit windows
I think that something is wrong with Indy
implementation of compression and the way
how -5 error should be interpreted but I'm
not sure. I tried some other pascal zlib libraries
and everyhing is good.
Used version is Windows CodeTyphon 5.1.0, Fpc = 2.7.1,
pl_Indy Source (24-10-2014 SVN Rev 5201)
I appreciate any help from Indy team or CodeTyphon team or
Lazarus team
*)
// *********************************************

begin
  try
    FavTestGZip;
  except
    on E: Exception do begin
      FavAppendToFile('Log.log', '; ' + FormatDateTime('yyyy-mm-dd hh:nn:ss.zzzzzz', Now) + '; ' + E.ClassName + '; ' + E.Message + #13#10);
      FavAppendToFile('Error.log', FormatDateTime('yyyy-mm-dd hh:nn:ss.zzzzzz', Now) + '; ' + E.ClassName + '; ' + E.Message + #13#10);
      raise
    end;
  end;
end.

